package tech.deepq.common.utils.signature;

import org.apache.commons.codec.binary.Base64;
import org.springframework.util.DigestUtils;
import tech.deepq.common.dto.SignatureConfig;
import tech.deepq.common.enums.SignatureEnumGroup;

import java.nio.charset.StandardCharsets;
import java.security.GeneralSecurityException;
import java.security.KeyFactory;
import java.security.Signature;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

/**
 * RSA
 *
 * @author yechuan
 * @since 2023/8/8 14:47
 **/

class RSASignatureService extends AbstractSignatureService {

    private RSASignatureService() {

    }

    public static AbstractSignatureService getInstance(SignatureConfig signatureConfig) throws GeneralSecurityException {
        RSASignatureService result = new RSASignatureService();
        result.init(signatureConfig);
        return result;
    }

    @Override
    public String signatureByPriKey(String content) throws GeneralSecurityException {
        if (signatureContext == null) {
            throw new RuntimeException("未初始化公私钥对信息");
        }
        if (signatureContext.getSignature() == null) {
            throw new RuntimeException("未初始化私钥信息");
        }
        Signature signature = signatureContext.getSignature();
        content = DigestUtils.md5DigestAsHex(content.getBytes(StandardCharsets.UTF_8)).toUpperCase();
        signature.update(content.getBytes(StandardCharsets.UTF_8));
        return Base64.encodeBase64String(signature.sign());
    }

    @Override
    void init(SignatureConfig signatureConfig) throws GeneralSecurityException {
        super.init(signatureConfig);

        String keyTypeName = signatureConfig.getKeyType();
        SignatureEnumGroup.KeyType keyType = SignatureEnumGroup.KeyType.valueOf(keyTypeName);

        KeyFactory keyFactory = KeyFactory.getInstance(SignatureEnumGroup.KeyAlgorithm.RSA.name());

        byte[] encodedKey = Base64.decodeBase64(signatureConfig.getKey());

        if (SignatureEnumGroup.KeyType.privateKey.equals(keyType)) {
            signatureContext.setPriKey(keyFactory.generatePrivate(new PKCS8EncodedKeySpec(encodedKey)));
            signatureContext.getSignature().initSign(signatureContext.getPriKey());
        } else if (SignatureEnumGroup.KeyType.publicKey.equals(keyType)) {
            signatureContext.setPubKey(keyFactory.generatePublic(new X509EncodedKeySpec(encodedKey)));
        }


    }


}
